# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, TypedDict

from .whatsapp_media_param import WhatsappMediaParam

__all__ = [
    "WhatsappInteractiveParam",
    "Action",
    "ActionButton",
    "ActionButtonReply",
    "ActionCard",
    "ActionCardAction",
    "ActionCardBody",
    "ActionCardHeader",
    "ActionParameters",
    "ActionSection",
    "ActionSectionProductItem",
    "ActionSectionRow",
    "Body",
    "Footer",
    "Header",
]


class ActionButtonReply(TypedDict, total=False):
    id: str
    """unique identifier for each button, 256 character maximum"""

    title: str
    """button label, 20 character maximum"""


class ActionButton(TypedDict, total=False):
    reply: ActionButtonReply

    type: Literal["reply"]


class ActionCardAction(TypedDict, total=False):
    catalog_id: str
    """the unique ID of the catalog"""

    product_retailer_id: str
    """the unique retailer ID of the product"""


class ActionCardBody(TypedDict, total=False):
    text: str
    """160 character maximum, up to 2 line breaks"""


class ActionCardHeader(TypedDict, total=False):
    image: WhatsappMediaParam

    type: Literal["image", "video"]

    video: WhatsappMediaParam


class ActionCard(TypedDict, total=False):
    action: ActionCardAction

    body: ActionCardBody

    card_index: int
    """unique index for each card (0-9)"""

    header: ActionCardHeader

    type: Literal["cta_url"]


class ActionParameters(TypedDict, total=False):
    display_text: str
    """button label text, 20 character maximum"""

    url: str
    """button URL to load when tapped by the user"""


class ActionSectionProductItem(TypedDict, total=False):
    product_retailer_id: str


class ActionSectionRow(TypedDict, total=False):
    id: str
    """arbitrary string identifying the row, 200 character maximum"""

    description: str
    """row description, 72 character maximum"""

    title: str
    """row title, 24 character maximum"""


class ActionSection(TypedDict, total=False):
    product_items: Iterable[ActionSectionProductItem]

    rows: Iterable[ActionSectionRow]

    title: str
    """section title, 24 character maximum"""


class Action(TypedDict, total=False):
    button: str

    buttons: Iterable[ActionButton]

    cards: Iterable[ActionCard]

    catalog_id: str

    mode: str

    name: str

    parameters: ActionParameters

    product_retailer_id: str

    sections: Iterable[ActionSection]


class Body(TypedDict, total=False):
    text: str
    """body text, 1024 character maximum"""


class Footer(TypedDict, total=False):
    text: str
    """footer text, 60 character maximum"""


class Header(TypedDict, total=False):
    document: WhatsappMediaParam

    image: WhatsappMediaParam

    sub_text: str

    text: str
    """header text, 60 character maximum"""

    video: WhatsappMediaParam


class WhatsappInteractiveParam(TypedDict, total=False):
    action: Action

    body: Body

    footer: Footer

    header: Header

    type: Literal["cta_url", "list", "carousel", "button", "location_request_message"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["PartnerCampaignUpdateParams"]


class PartnerCampaignUpdateParams(TypedDict, total=False):
    webhook_failover_url: Annotated[str, PropertyInfo(alias="webhookFailoverURL")]
    """Webhook failover to which campaign status updates are sent."""

    webhook_url: Annotated[str, PropertyInfo(alias="webhookURL")]
    """Webhook to which campaign status updates are sent."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .requests import (
    RequestsResource,
    AsyncRequestsResource,
    RequestsResourceWithRawResponse,
    AsyncRequestsResourceWithRawResponse,
    RequestsResourceWithStreamingResponse,
    AsyncRequestsResourceWithStreamingResponse,
)
from .verification import (
    VerificationResource,
    AsyncVerificationResource,
    VerificationResourceWithRawResponse,
    AsyncVerificationResourceWithRawResponse,
    VerificationResourceWithStreamingResponse,
    AsyncVerificationResourceWithStreamingResponse,
)

__all__ = [
    "RequestsResource",
    "AsyncRequestsResource",
    "RequestsResourceWithRawResponse",
    "AsyncRequestsResourceWithRawResponse",
    "RequestsResourceWithStreamingResponse",
    "AsyncRequestsResourceWithStreamingResponse",
    "VerificationResource",
    "AsyncVerificationResource",
    "VerificationResourceWithRawResponse",
    "AsyncVerificationResourceWithRawResponse",
    "VerificationResourceWithStreamingResponse",
    "AsyncVerificationResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["OAuthRegisterResponse"]


class OAuthRegisterResponse(BaseModel):
    client_id: str
    """Unique client identifier"""

    client_id_issued_at: int
    """Unix timestamp of when the client ID was issued"""

    client_name: Optional[str] = None
    """Human-readable client name"""

    client_secret: Optional[str] = None
    """Client secret (only for confidential clients)"""

    grant_types: Optional[List[str]] = None
    """Array of allowed grant types"""

    logo_uri: Optional[str] = None
    """URL of the client logo"""

    policy_uri: Optional[str] = None
    """URL of the client's privacy policy"""

    redirect_uris: Optional[List[str]] = None
    """Array of redirection URIs"""

    response_types: Optional[List[str]] = None
    """Array of allowed response types"""

    scope: Optional[str] = None
    """Space-separated scope values"""

    token_endpoint_auth_method: Optional[str] = None
    """Token endpoint authentication method"""

    tos_uri: Optional[str] = None
    """URL of the client's terms of service"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncDefaultPaginationForQueues, AsyncDefaultPaginationForQueues
from ...._base_client import AsyncPaginator, make_request_options
from ....types.texml.accounts import queue_list_params, queue_create_params, queue_update_params
from ....types.texml.accounts.queue_list_response import QueueListResponse
from ....types.texml.accounts.queue_create_response import QueueCreateResponse
from ....types.texml.accounts.queue_update_response import QueueUpdateResponse
from ....types.texml.accounts.queue_retrieve_response import QueueRetrieveResponse

__all__ = ["QueuesResource", "AsyncQueuesResource"]


class QueuesResource(SyncAPIResource):
    """TeXML REST Commands"""

    @cached_property
    def with_raw_response(self) -> QueuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return QueuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QueuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return QueuesResourceWithStreamingResponse(self)

    def create(
        self,
        account_sid: str,
        *,
        friendly_name: str | Omit = omit,
        max_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueCreateResponse:
        """
        Creates a new queue resource.

        Args:
          friendly_name: A human readable name for the queue.

          max_size: The maximum size of the queue.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        return self._post(
            f"/texml/Accounts/{account_sid}/Queues",
            body=maybe_transform(
                {
                    "friendly_name": friendly_name,
                    "max_size": max_size,
                },
                queue_create_params.QueueCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueCreateResponse,
        )

    def retrieve(
        self,
        queue_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueRetrieveResponse:
        """
        Returns a queue resource.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not queue_sid:
            raise ValueError(f"Expected a non-empty value for `queue_sid` but received {queue_sid!r}")
        return self._get(
            f"/texml/Accounts/{account_sid}/Queues/{queue_sid}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueRetrieveResponse,
        )

    def update(
        self,
        queue_sid: str,
        *,
        account_sid: str,
        max_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueUpdateResponse:
        """
        Updates a queue resource.

        Args:
          max_size: The maximum size of the queue.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not queue_sid:
            raise ValueError(f"Expected a non-empty value for `queue_sid` but received {queue_sid!r}")
        return self._post(
            f"/texml/Accounts/{account_sid}/Queues/{queue_sid}",
            body=maybe_transform({"max_size": max_size}, queue_update_params.QueueUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueUpdateResponse,
        )

    def list(
        self,
        account_sid: str,
        *,
        date_created: str | Omit = omit,
        date_updated: str | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPaginationForQueues[QueueListResponse]:
        """
        Lists queue resources.

        Args:
          date_created: Filters conferences by the creation date. Expected format is YYYY-MM-DD. Also
              accepts inequality operators, e.g. DateCreated>=2023-05-22.

          date_updated: Filters conferences by the time they were last updated. Expected format is
              YYYY-MM-DD. Also accepts inequality operators, e.g. DateUpdated>=2023-05-22.

          page: The number of the page to be displayed, zero-indexed, should be used in
              conjuction with PageToken.

          page_size: The number of records to be displayed on a page

          page_token: Used to request the next page of results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        return self._get_api_list(
            f"/texml/Accounts/{account_sid}/Queues",
            page=SyncDefaultPaginationForQueues[QueueListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_created": date_created,
                        "date_updated": date_updated,
                        "page": page,
                        "page_size": page_size,
                        "page_token": page_token,
                    },
                    queue_list_params.QueueListParams,
                ),
            ),
            model=QueueListResponse,
        )

    def delete(
        self,
        queue_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a queue resource.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not queue_sid:
            raise ValueError(f"Expected a non-empty value for `queue_sid` but received {queue_sid!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/texml/Accounts/{account_sid}/Queues/{queue_sid}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncQueuesResource(AsyncAPIResource):
    """TeXML REST Commands"""

    @cached_property
    def with_raw_response(self) -> AsyncQueuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncQueuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQueuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncQueuesResourceWithStreamingResponse(self)

    async def create(
        self,
        account_sid: str,
        *,
        friendly_name: str | Omit = omit,
        max_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueCreateResponse:
        """
        Creates a new queue resource.

        Args:
          friendly_name: A human readable name for the queue.

          max_size: The maximum size of the queue.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        return await self._post(
            f"/texml/Accounts/{account_sid}/Queues",
            body=await async_maybe_transform(
                {
                    "friendly_name": friendly_name,
                    "max_size": max_size,
                },
                queue_create_params.QueueCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueCreateResponse,
        )

    async def retrieve(
        self,
        queue_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueRetrieveResponse:
        """
        Returns a queue resource.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not queue_sid:
            raise ValueError(f"Expected a non-empty value for `queue_sid` but received {queue_sid!r}")
        return await self._get(
            f"/texml/Accounts/{account_sid}/Queues/{queue_sid}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueRetrieveResponse,
        )

    async def update(
        self,
        queue_sid: str,
        *,
        account_sid: str,
        max_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueUpdateResponse:
        """
        Updates a queue resource.

        Args:
          max_size: The maximum size of the queue.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not queue_sid:
            raise ValueError(f"Expected a non-empty value for `queue_sid` but received {queue_sid!r}")
        return await self._post(
            f"/texml/Accounts/{account_sid}/Queues/{queue_sid}",
            body=await async_maybe_transform({"max_size": max_size}, queue_update_params.QueueUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueUpdateResponse,
        )

    def list(
        self,
        account_sid: str,
        *,
        date_created: str | Omit = omit,
        date_updated: str | Omit = omit,
        page: int | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[QueueListResponse, AsyncDefaultPaginationForQueues[QueueListResponse]]:
        """
        Lists queue resources.

        Args:
          date_created: Filters conferences by the creation date. Expected format is YYYY-MM-DD. Also
              accepts inequality operators, e.g. DateCreated>=2023-05-22.

          date_updated: Filters conferences by the time they were last updated. Expected format is
              YYYY-MM-DD. Also accepts inequality operators, e.g. DateUpdated>=2023-05-22.

          page: The number of the page to be displayed, zero-indexed, should be used in
              conjuction with PageToken.

          page_size: The number of records to be displayed on a page

          page_token: Used to request the next page of results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        return self._get_api_list(
            f"/texml/Accounts/{account_sid}/Queues",
            page=AsyncDefaultPaginationForQueues[QueueListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_created": date_created,
                        "date_updated": date_updated,
                        "page": page,
                        "page_size": page_size,
                        "page_token": page_token,
                    },
                    queue_list_params.QueueListParams,
                ),
            ),
            model=QueueListResponse,
        )

    async def delete(
        self,
        queue_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a queue resource.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not queue_sid:
            raise ValueError(f"Expected a non-empty value for `queue_sid` but received {queue_sid!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/texml/Accounts/{account_sid}/Queues/{queue_sid}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class QueuesResourceWithRawResponse:
    def __init__(self, queues: QueuesResource) -> None:
        self._queues = queues

        self.create = to_raw_response_wrapper(
            queues.create,
        )
        self.retrieve = to_raw_response_wrapper(
            queues.retrieve,
        )
        self.update = to_raw_response_wrapper(
            queues.update,
        )
        self.list = to_raw_response_wrapper(
            queues.list,
        )
        self.delete = to_raw_response_wrapper(
            queues.delete,
        )


class AsyncQueuesResourceWithRawResponse:
    def __init__(self, queues: AsyncQueuesResource) -> None:
        self._queues = queues

        self.create = async_to_raw_response_wrapper(
            queues.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            queues.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            queues.update,
        )
        self.list = async_to_raw_response_wrapper(
            queues.list,
        )
        self.delete = async_to_raw_response_wrapper(
            queues.delete,
        )


class QueuesResourceWithStreamingResponse:
    def __init__(self, queues: QueuesResource) -> None:
        self._queues = queues

        self.create = to_streamed_response_wrapper(
            queues.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            queues.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            queues.update,
        )
        self.list = to_streamed_response_wrapper(
            queues.list,
        )
        self.delete = to_streamed_response_wrapper(
            queues.delete,
        )


class AsyncQueuesResourceWithStreamingResponse:
    def __init__(self, queues: AsyncQueuesResource) -> None:
        self._queues = queues

        self.create = async_to_streamed_response_wrapper(
            queues.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            queues.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            queues.update,
        )
        self.list = async_to_streamed_response_wrapper(
            queues.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            queues.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["TextToSpeechGenerateResponse"]


class TextToSpeechGenerateResponse(BaseModel):
    """Response when `output_type` is `base64_output`."""

    base64_audio: Optional[str] = None
    """Base64-encoded audio data."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["MobilePhoneNumberUpdateParams", "CallForwarding", "CallRecording", "CnamListing", "Inbound", "Outbound"]


class MobilePhoneNumberUpdateParams(TypedDict, total=False):
    call_forwarding: CallForwarding

    call_recording: CallRecording

    caller_id_name_enabled: bool

    cnam_listing: CnamListing

    connection_id: Optional[str]

    customer_reference: Optional[str]

    inbound: Inbound

    inbound_call_screening: Literal["disabled", "reject_calls", "flag_calls"]

    noise_suppression: bool

    outbound: Outbound

    tags: SequenceNotStr[str]


class CallForwarding(TypedDict, total=False):
    call_forwarding_enabled: bool

    forwarding_type: Optional[Literal["always", "on-failure"]]

    forwards_to: Optional[str]


class CallRecording(TypedDict, total=False):
    inbound_call_recording_channels: Literal["single", "dual"]

    inbound_call_recording_enabled: bool

    inbound_call_recording_format: Literal["wav", "mp3"]


class CnamListing(TypedDict, total=False):
    cnam_listing_details: Optional[str]

    cnam_listing_enabled: bool


class Inbound(TypedDict, total=False):
    interception_app_id: Optional[str]
    """
    The ID of the CallControl or TeXML Application that will intercept inbound
    calls.
    """


class Outbound(TypedDict, total=False):
    interception_app_id: Optional[str]
    """
    The ID of the CallControl or TeXML Application that will intercept outbound
    calls.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TexmlSecretsParams"]


class TexmlSecretsParams(TypedDict, total=False):
    name: Required[str]
    """
    Name used as a reference for the secret, if the name already exists within the
    account its value will be replaced
    """

    value: Required[str]
    """Secret value which will be used when rendering the TeXML template"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo

__all__ = ["VoiceCreateParams"]


class VoiceCreateParams(TypedDict, total=False):
    end_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """End time in ISO format"""

    start_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """Start time in ISO format"""

    aggregation_type: int
    """
    Aggregation type: All = 0, By Connections = 1, By Tags = 2, By Billing Group = 3
    """

    connections: Iterable[int]
    """List of connections to filter by"""

    managed_accounts: SequenceNotStr[str]
    """List of managed accounts to include"""

    product_breakdown: int
    """
    Product breakdown type: No breakdown = 0, DID vs Toll-free = 1, Country = 2, DID
    vs Toll-free per Country = 3
    """

    select_all_managed_accounts: bool
    """Whether to select all managed accounts"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["CredentialOutbound"]


class CredentialOutbound(BaseModel):
    ani_override: Optional[str] = None
    """
    Set a phone number as the ani_override value to override caller id number on
    outbound calls.
    """

    ani_override_type: Optional[Literal["always", "normal", "emergency"]] = None
    """Specifies when we apply your ani_override setting.

    Only applies when ani_override is not blank.
    """

    call_parking_enabled: Optional[bool] = None
    """
    Forces all SIP calls originated on this connection to be "parked" instead of
    "bridged" to the destination specified on the URI. Parked calls will return
    ringback to the caller and will await for a Call Control command to define which
    action will be taken next.
    """

    channel_limit: Optional[int] = None
    """
    When set, this will limit the total number of outbound calls to phone numbers
    associated with this connection.
    """

    generate_ringback_tone: Optional[bool] = None
    """Generate ringback tone through 183 session progress message with early media."""

    instant_ringback_enabled: Optional[bool] = None
    """
    When set, ringback will not wait for indication before sending ringback tone to
    calling party.
    """

    localization: Optional[str] = None
    """
    A 2-character country code specifying the country whose national dialing rules
    should be used. For example, if set to `US` then any US number can be dialed
    without preprending +1 to the number. When left blank, Telnyx will try US and GB
    dialing rules, in that order, by default.
    """

    outbound_voice_profile_id: Optional[str] = None
    """Identifies the associated outbound voice profile."""

    t38_reinvite_source: Optional[
        Literal["telnyx", "customer", "disabled", "passthru", "caller-passthru", "callee-passthru"]
    ] = None
    """This setting only affects connections with Fax-type Outbound Voice Profiles.

    The setting dictates whether or not Telnyx sends a t.38 reinvite.<br/><br/> By
    default, Telnyx will send the re-invite. If set to `customer`, the caller is
    expected to send the t.38 reinvite.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ....._types import FileTypes

__all__ = ["PhotoUploadParams"]


class PhotoUploadParams(TypedDict, total=False):
    file: Required[FileTypes]
    """Image file (JPEG recommended, max 10 MB)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .upload import Upload
from ..._models import BaseModel

__all__ = ["UploadRetryResponse"]


class UploadRetryResponse(BaseModel):
    data: Optional[Upload] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .user_bundles import (
    UserBundlesResource,
    AsyncUserBundlesResource,
    UserBundlesResourceWithRawResponse,
    AsyncUserBundlesResourceWithRawResponse,
    UserBundlesResourceWithStreamingResponse,
    AsyncUserBundlesResourceWithStreamingResponse,
)
from .bundle_pricing import (
    BundlePricingResource,
    AsyncBundlePricingResource,
    BundlePricingResourceWithRawResponse,
    AsyncBundlePricingResourceWithRawResponse,
    BundlePricingResourceWithStreamingResponse,
    AsyncBundlePricingResourceWithStreamingResponse,
)
from .billing_bundles import (
    BillingBundlesResource,
    AsyncBillingBundlesResource,
    BillingBundlesResourceWithRawResponse,
    AsyncBillingBundlesResourceWithRawResponse,
    BillingBundlesResourceWithStreamingResponse,
    AsyncBillingBundlesResourceWithStreamingResponse,
)

__all__ = [
    "BillingBundlesResource",
    "AsyncBillingBundlesResource",
    "BillingBundlesResourceWithRawResponse",
    "AsyncBillingBundlesResourceWithRawResponse",
    "BillingBundlesResourceWithStreamingResponse",
    "AsyncBillingBundlesResourceWithStreamingResponse",
    "UserBundlesResource",
    "AsyncUserBundlesResource",
    "UserBundlesResourceWithRawResponse",
    "AsyncUserBundlesResourceWithRawResponse",
    "UserBundlesResourceWithStreamingResponse",
    "AsyncUserBundlesResourceWithStreamingResponse",
    "BundlePricingResource",
    "AsyncBundlePricingResource",
    "BundlePricingResourceWithRawResponse",
    "AsyncBundlePricingResourceWithRawResponse",
    "BundlePricingResourceWithStreamingResponse",
    "AsyncBundlePricingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    InexplicitNumberOrderResponse,
    InexplicitNumberOrderCreateResponse,
    InexplicitNumberOrderRetrieveResponse,
)
from telnyx.pagination import (
    SyncDefaultFlatPaginationForInexplicitNumberOrders,
    AsyncDefaultFlatPaginationForInexplicitNumberOrders,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestInexplicitNumberOrders:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        inexplicit_number_order = client.inexplicit_number_orders.create(
            ordering_groups=[
                {
                    "count_requested": "count_requested",
                    "country_iso": "US",
                    "phone_number_type": "phone_number_type",
                }
            ],
        )
        assert_matches_type(InexplicitNumberOrderCreateResponse, inexplicit_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        inexplicit_number_order = client.inexplicit_number_orders.create(
            ordering_groups=[
                {
                    "count_requested": "count_requested",
                    "country_iso": "US",
                    "phone_number_type": "phone_number_type",
                    "administrative_area": "administrative_area",
                    "exclude_held_numbers": True,
                    "features": ["string"],
                    "locality": "locality",
                    "national_destination_code": "national_destination_code",
                    "phone_number": {
                        "contains": "contains",
                        "ends_with": "ends_with",
                        "starts_with": "starts_with",
                    },
                    "quickship": True,
                    "strategy": "always",
                }
            ],
            billing_group_id="billing_group_id",
            connection_id="connection_id",
            customer_reference="customer_reference",
            messaging_profile_id="messaging_profile_id",
        )
        assert_matches_type(InexplicitNumberOrderCreateResponse, inexplicit_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.inexplicit_number_orders.with_raw_response.create(
            ordering_groups=[
                {
                    "count_requested": "count_requested",
                    "country_iso": "US",
                    "phone_number_type": "phone_number_type",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inexplicit_number_order = response.parse()
        assert_matches_type(InexplicitNumberOrderCreateResponse, inexplicit_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.inexplicit_number_orders.with_streaming_response.create(
            ordering_groups=[
                {
                    "count_requested": "count_requested",
                    "country_iso": "US",
                    "phone_number_type": "phone_number_type",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inexplicit_number_order = response.parse()
            assert_matches_type(InexplicitNumberOrderCreateResponse, inexplicit_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        inexplicit_number_order = client.inexplicit_number_orders.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(InexplicitNumberOrderRetrieveResponse, inexplicit_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.inexplicit_number_orders.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inexplicit_number_order = response.parse()
        assert_matches_type(InexplicitNumberOrderRetrieveResponse, inexplicit_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.inexplicit_number_orders.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inexplicit_number_order = response.parse()
            assert_matches_type(InexplicitNumberOrderRetrieveResponse, inexplicit_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.inexplicit_number_orders.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        inexplicit_number_order = client.inexplicit_number_orders.list()
        assert_matches_type(
            SyncDefaultFlatPaginationForInexplicitNumberOrders[InexplicitNumberOrderResponse],
            inexplicit_number_order,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        inexplicit_number_order = client.inexplicit_number_orders.list(
            page_number=1,
            page_size=1,
        )
        assert_matches_type(
            SyncDefaultFlatPaginationForInexplicitNumberOrders[InexplicitNumberOrderResponse],
            inexplicit_number_order,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.inexplicit_number_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inexplicit_number_order = response.parse()
        assert_matches_type(
            SyncDefaultFlatPaginationForInexplicitNumberOrders[InexplicitNumberOrderResponse],
            inexplicit_number_order,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.inexplicit_number_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inexplicit_number_order = response.parse()
            assert_matches_type(
                SyncDefaultFlatPaginationForInexplicitNumberOrders[InexplicitNumberOrderResponse],
                inexplicit_number_order,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True


class TestAsyncInexplicitNumberOrders:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        inexplicit_number_order = await async_client.inexplicit_number_orders.create(
            ordering_groups=[
                {
                    "count_requested": "count_requested",
                    "country_iso": "US",
                    "phone_number_type": "phone_number_type",
                }
            ],
        )
        assert_matches_type(InexplicitNumberOrderCreateResponse, inexplicit_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        inexplicit_number_order = await async_client.inexplicit_number_orders.create(
            ordering_groups=[
                {
                    "count_requested": "count_requested",
                    "country_iso": "US",
                    "phone_number_type": "phone_number_type",
                    "administrative_area": "administrative_area",
                    "exclude_held_numbers": True,
                    "features": ["string"],
                    "locality": "locality",
                    "national_destination_code": "national_destination_code",
                    "phone_number": {
                        "contains": "contains",
                        "ends_with": "ends_with",
                        "starts_with": "starts_with",
                    },
                    "quickship": True,
                    "strategy": "always",
                }
            ],
            billing_group_id="billing_group_id",
            connection_id="connection_id",
            customer_reference="customer_reference",
            messaging_profile_id="messaging_profile_id",
        )
        assert_matches_type(InexplicitNumberOrderCreateResponse, inexplicit_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.inexplicit_number_orders.with_raw_response.create(
            ordering_groups=[
                {
                    "count_requested": "count_requested",
                    "country_iso": "US",
                    "phone_number_type": "phone_number_type",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inexplicit_number_order = await response.parse()
        assert_matches_type(InexplicitNumberOrderCreateResponse, inexplicit_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.inexplicit_number_orders.with_streaming_response.create(
            ordering_groups=[
                {
                    "count_requested": "count_requested",
                    "country_iso": "US",
                    "phone_number_type": "phone_number_type",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inexplicit_number_order = await response.parse()
            assert_matches_type(InexplicitNumberOrderCreateResponse, inexplicit_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        inexplicit_number_order = await async_client.inexplicit_number_orders.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(InexplicitNumberOrderRetrieveResponse, inexplicit_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.inexplicit_number_orders.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inexplicit_number_order = await response.parse()
        assert_matches_type(InexplicitNumberOrderRetrieveResponse, inexplicit_number_order, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.inexplicit_number_orders.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inexplicit_number_order = await response.parse()
            assert_matches_type(InexplicitNumberOrderRetrieveResponse, inexplicit_number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.inexplicit_number_orders.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        inexplicit_number_order = await async_client.inexplicit_number_orders.list()
        assert_matches_type(
            AsyncDefaultFlatPaginationForInexplicitNumberOrders[InexplicitNumberOrderResponse],
            inexplicit_number_order,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        inexplicit_number_order = await async_client.inexplicit_number_orders.list(
            page_number=1,
            page_size=1,
        )
        assert_matches_type(
            AsyncDefaultFlatPaginationForInexplicitNumberOrders[InexplicitNumberOrderResponse],
            inexplicit_number_order,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.inexplicit_number_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inexplicit_number_order = await response.parse()
        assert_matches_type(
            AsyncDefaultFlatPaginationForInexplicitNumberOrders[InexplicitNumberOrderResponse],
            inexplicit_number_order,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.inexplicit_number_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inexplicit_number_order = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPaginationForInexplicitNumberOrders[InexplicitNumberOrderResponse],
                inexplicit_number_order,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["RoomParticipant"]


class RoomParticipant(BaseModel):
    id: Optional[str] = None
    """A unique identifier for the room participant."""

    context: Optional[str] = None
    """Context provided to the given participant through the client SDK"""

    joined_at: Optional[datetime] = None
    """ISO 8601 timestamp when the participant joined the session."""

    left_at: Optional[datetime] = None
    """ISO 8601 timestamp when the participant left the session."""

    record_type: Optional[str] = None

    session_id: Optional[str] = None
    """Identify the room session that participant is part of."""

    updated_at: Optional[datetime] = None
    """ISO 8601 timestamp when the participant was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["RequirementGroupCreateParams", "RegulatoryRequirement"]


class RequirementGroupCreateParams(TypedDict, total=False):
    action: Required[Literal["ordering", "porting"]]

    country_code: Required[str]
    """ISO alpha 2 country code"""

    phone_number_type: Required[Literal["local", "toll_free", "mobile", "national", "shared_cost"]]

    customer_reference: str

    regulatory_requirements: Iterable[RegulatoryRequirement]


class RegulatoryRequirement(TypedDict, total=False):
    field_value: str

    requirement_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .messaging_profile import MessagingProfile

__all__ = ["MessagingProfileUpdateResponse"]


class MessagingProfileUpdateResponse(BaseModel):
    data: Optional[MessagingProfile] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from ...types import portout_list_params, portout_update_status_params, portout_list_rejection_codes_params
from .reports import (
    ReportsResource,
    AsyncReportsResource,
    ReportsResourceWithRawResponse,
    AsyncReportsResourceWithRawResponse,
    ReportsResourceWithStreamingResponse,
    AsyncReportsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .comments import (
    CommentsResource,
    AsyncCommentsResource,
    CommentsResourceWithRawResponse,
    AsyncCommentsResourceWithRawResponse,
    CommentsResourceWithStreamingResponse,
    AsyncCommentsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from .supporting_documents import (
    SupportingDocumentsResource,
    AsyncSupportingDocumentsResource,
    SupportingDocumentsResourceWithRawResponse,
    AsyncSupportingDocumentsResourceWithRawResponse,
    SupportingDocumentsResourceWithStreamingResponse,
    AsyncSupportingDocumentsResourceWithStreamingResponse,
)
from ...types.portout_details import PortoutDetails
from ...types.portout_retrieve_response import PortoutRetrieveResponse
from ...types.portout_update_status_response import PortoutUpdateStatusResponse
from ...types.portout_list_rejection_codes_response import PortoutListRejectionCodesResponse

__all__ = ["PortoutsResource", "AsyncPortoutsResource"]


class PortoutsResource(SyncAPIResource):
    """Number portout operations"""

    @cached_property
    def events(self) -> EventsResource:
        """Number portout operations"""
        return EventsResource(self._client)

    @cached_property
    def reports(self) -> ReportsResource:
        """Number portout operations"""
        return ReportsResource(self._client)

    @cached_property
    def comments(self) -> CommentsResource:
        """Number portout operations"""
        return CommentsResource(self._client)

    @cached_property
    def supporting_documents(self) -> SupportingDocumentsResource:
        """Number portout operations"""
        return SupportingDocumentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> PortoutsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PortoutsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PortoutsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PortoutsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortoutRetrieveResponse:
        """
        Returns the portout request based on the ID provided

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/portouts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PortoutRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: portout_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[PortoutDetails]:
        """
        Returns the portout requests according to filters

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[carrier_name], filter[country_code], filter[country_code_in],
              filter[foc_date], filter[inserted_at], filter[phone_number], filter[pon],
              filter[ported_out_at], filter[spid], filter[status], filter[status_in],
              filter[support_key]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/portouts",
            page=SyncDefaultFlatPagination[PortoutDetails],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    portout_list_params.PortoutListParams,
                ),
            ),
            model=PortoutDetails,
        )

    def list_rejection_codes(
        self,
        portout_id: str,
        *,
        filter: portout_list_rejection_codes_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortoutListRejectionCodesResponse:
        """
        Given a port-out ID, list rejection codes that are eligible for that port-out

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[code],
              filter[code][in]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not portout_id:
            raise ValueError(f"Expected a non-empty value for `portout_id` but received {portout_id!r}")
        return self._get(
            f"/portouts/rejections/{portout_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"filter": filter}, portout_list_rejection_codes_params.PortoutListRejectionCodesParams
                ),
            ),
            cast_to=PortoutListRejectionCodesResponse,
        )

    def update_status(
        self,
        status: Literal["authorized", "rejected-pending"],
        *,
        id: str,
        reason: str,
        host_messaging: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortoutUpdateStatusResponse:
        """
        Authorize or reject portout request

        Args:
          reason: Provide a reason if rejecting the port out request

          host_messaging: Indicates whether messaging services should be maintained with Telnyx after the
              port out completes

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not status:
            raise ValueError(f"Expected a non-empty value for `status` but received {status!r}")
        return self._patch(
            f"/portouts/{id}/{status}",
            body=maybe_transform(
                {
                    "reason": reason,
                    "host_messaging": host_messaging,
                },
                portout_update_status_params.PortoutUpdateStatusParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PortoutUpdateStatusResponse,
        )


class AsyncPortoutsResource(AsyncAPIResource):
    """Number portout operations"""

    @cached_property
    def events(self) -> AsyncEventsResource:
        """Number portout operations"""
        return AsyncEventsResource(self._client)

    @cached_property
    def reports(self) -> AsyncReportsResource:
        """Number portout operations"""
        return AsyncReportsResource(self._client)

    @cached_property
    def comments(self) -> AsyncCommentsResource:
        """Number portout operations"""
        return AsyncCommentsResource(self._client)

    @cached_property
    def supporting_documents(self) -> AsyncSupportingDocumentsResource:
        """Number portout operations"""
        return AsyncSupportingDocumentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPortoutsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPortoutsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPortoutsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPortoutsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortoutRetrieveResponse:
        """
        Returns the portout request based on the ID provided

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/portouts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PortoutRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: portout_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PortoutDetails, AsyncDefaultFlatPagination[PortoutDetails]]:
        """
        Returns the portout requests according to filters

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[carrier_name], filter[country_code], filter[country_code_in],
              filter[foc_date], filter[inserted_at], filter[phone_number], filter[pon],
              filter[ported_out_at], filter[spid], filter[status], filter[status_in],
              filter[support_key]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/portouts",
            page=AsyncDefaultFlatPagination[PortoutDetails],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    portout_list_params.PortoutListParams,
                ),
            ),
            model=PortoutDetails,
        )

    async def list_rejection_codes(
        self,
        portout_id: str,
        *,
        filter: portout_list_rejection_codes_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortoutListRejectionCodesResponse:
        """
        Given a port-out ID, list rejection codes that are eligible for that port-out

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[code],
              filter[code][in]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not portout_id:
            raise ValueError(f"Expected a non-empty value for `portout_id` but received {portout_id!r}")
        return await self._get(
            f"/portouts/rejections/{portout_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"filter": filter}, portout_list_rejection_codes_params.PortoutListRejectionCodesParams
                ),
            ),
            cast_to=PortoutListRejectionCodesResponse,
        )

    async def update_status(
        self,
        status: Literal["authorized", "rejected-pending"],
        *,
        id: str,
        reason: str,
        host_messaging: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortoutUpdateStatusResponse:
        """
        Authorize or reject portout request

        Args:
          reason: Provide a reason if rejecting the port out request

          host_messaging: Indicates whether messaging services should be maintained with Telnyx after the
              port out completes

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not status:
            raise ValueError(f"Expected a non-empty value for `status` but received {status!r}")
        return await self._patch(
            f"/portouts/{id}/{status}",
            body=await async_maybe_transform(
                {
                    "reason": reason,
                    "host_messaging": host_messaging,
                },
                portout_update_status_params.PortoutUpdateStatusParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PortoutUpdateStatusResponse,
        )


class PortoutsResourceWithRawResponse:
    def __init__(self, portouts: PortoutsResource) -> None:
        self._portouts = portouts

        self.retrieve = to_raw_response_wrapper(
            portouts.retrieve,
        )
        self.list = to_raw_response_wrapper(
            portouts.list,
        )
        self.list_rejection_codes = to_raw_response_wrapper(
            portouts.list_rejection_codes,
        )
        self.update_status = to_raw_response_wrapper(
            portouts.update_status,
        )

    @cached_property
    def events(self) -> EventsResourceWithRawResponse:
        """Number portout operations"""
        return EventsResourceWithRawResponse(self._portouts.events)

    @cached_property
    def reports(self) -> ReportsResourceWithRawResponse:
        """Number portout operations"""
        return ReportsResourceWithRawResponse(self._portouts.reports)

    @cached_property
    def comments(self) -> CommentsResourceWithRawResponse:
        """Number portout operations"""
        return CommentsResourceWithRawResponse(self._portouts.comments)

    @cached_property
    def supporting_documents(self) -> SupportingDocumentsResourceWithRawResponse:
        """Number portout operations"""
        return SupportingDocumentsResourceWithRawResponse(self._portouts.supporting_documents)


class AsyncPortoutsResourceWithRawResponse:
    def __init__(self, portouts: AsyncPortoutsResource) -> None:
        self._portouts = portouts

        self.retrieve = async_to_raw_response_wrapper(
            portouts.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            portouts.list,
        )
        self.list_rejection_codes = async_to_raw_response_wrapper(
            portouts.list_rejection_codes,
        )
        self.update_status = async_to_raw_response_wrapper(
            portouts.update_status,
        )

    @cached_property
    def events(self) -> AsyncEventsResourceWithRawResponse:
        """Number portout operations"""
        return AsyncEventsResourceWithRawResponse(self._portouts.events)

    @cached_property
    def reports(self) -> AsyncReportsResourceWithRawResponse:
        """Number portout operations"""
        return AsyncReportsResourceWithRawResponse(self._portouts.reports)

    @cached_property
    def comments(self) -> AsyncCommentsResourceWithRawResponse:
        """Number portout operations"""
        return AsyncCommentsResourceWithRawResponse(self._portouts.comments)

    @cached_property
    def supporting_documents(self) -> AsyncSupportingDocumentsResourceWithRawResponse:
        """Number portout operations"""
        return AsyncSupportingDocumentsResourceWithRawResponse(self._portouts.supporting_documents)


class PortoutsResourceWithStreamingResponse:
    def __init__(self, portouts: PortoutsResource) -> None:
        self._portouts = portouts

        self.retrieve = to_streamed_response_wrapper(
            portouts.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            portouts.list,
        )
        self.list_rejection_codes = to_streamed_response_wrapper(
            portouts.list_rejection_codes,
        )
        self.update_status = to_streamed_response_wrapper(
            portouts.update_status,
        )

    @cached_property
    def events(self) -> EventsResourceWithStreamingResponse:
        """Number portout operations"""
        return EventsResourceWithStreamingResponse(self._portouts.events)

    @cached_property
    def reports(self) -> ReportsResourceWithStreamingResponse:
        """Number portout operations"""
        return ReportsResourceWithStreamingResponse(self._portouts.reports)

    @cached_property
    def comments(self) -> CommentsResourceWithStreamingResponse:
        """Number portout operations"""
        return CommentsResourceWithStreamingResponse(self._portouts.comments)

    @cached_property
    def supporting_documents(self) -> SupportingDocumentsResourceWithStreamingResponse:
        """Number portout operations"""
        return SupportingDocumentsResourceWithStreamingResponse(self._portouts.supporting_documents)


class AsyncPortoutsResourceWithStreamingResponse:
    def __init__(self, portouts: AsyncPortoutsResource) -> None:
        self._portouts = portouts

        self.retrieve = async_to_streamed_response_wrapper(
            portouts.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            portouts.list,
        )
        self.list_rejection_codes = async_to_streamed_response_wrapper(
            portouts.list_rejection_codes,
        )
        self.update_status = async_to_streamed_response_wrapper(
            portouts.update_status,
        )

    @cached_property
    def events(self) -> AsyncEventsResourceWithStreamingResponse:
        """Number portout operations"""
        return AsyncEventsResourceWithStreamingResponse(self._portouts.events)

    @cached_property
    def reports(self) -> AsyncReportsResourceWithStreamingResponse:
        """Number portout operations"""
        return AsyncReportsResourceWithStreamingResponse(self._portouts.reports)

    @cached_property
    def comments(self) -> AsyncCommentsResourceWithStreamingResponse:
        """Number portout operations"""
        return AsyncCommentsResourceWithStreamingResponse(self._portouts.comments)

    @cached_property
    def supporting_documents(self) -> AsyncSupportingDocumentsResourceWithStreamingResponse:
        """Number portout operations"""
        return AsyncSupportingDocumentsResourceWithStreamingResponse(self._portouts.supporting_documents)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["RcsAgent"]


class RcsAgent(BaseModel):
    agent_id: Optional[str] = None
    """RCS Agent ID"""

    agent_name: Optional[str] = None
    """Human readable agent name"""

    created_at: Optional[datetime] = None
    """Date and time when the resource was created"""

    enabled: Optional[bool] = None
    """Specifies whether the agent is enabled"""

    profile_id: Optional[str] = None
    """Messaging profile ID associated with the RCS Agent"""

    updated_at: Optional[datetime] = None
    """Date and time when the resource was updated"""

    user_id: Optional[str] = None
    """User ID associated with the RCS Agent"""

    webhook_failover_url: Optional[str] = None
    """Failover URL to receive RCS events"""

    webhook_url: Optional[str] = None
    """URL to receive RCS events"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.ai import chat_create_completion_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.ai.chat_create_completion_response import ChatCreateCompletionResponse

__all__ = ["ChatResource", "AsyncChatResource"]


class ChatResource(SyncAPIResource):
    """Generate text with LLMs"""

    @cached_property
    def with_raw_response(self) -> ChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ChatResourceWithStreamingResponse(self)

    def create_completion(
        self,
        *,
        messages: Iterable[chat_create_completion_params.Message],
        api_key_ref: str | Omit = omit,
        best_of: int | Omit = omit,
        early_stopping: bool | Omit = omit,
        enable_thinking: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        guided_choice: SequenceNotStr[str] | Omit = omit,
        guided_json: Dict[str, object] | Omit = omit,
        guided_regex: str | Omit = omit,
        length_penalty: float | Omit = omit,
        logprobs: bool | Omit = omit,
        max_tokens: int | Omit = omit,
        min_p: float | Omit = omit,
        model: str | Omit = omit,
        n: float | Omit = omit,
        presence_penalty: float | Omit = omit,
        response_format: chat_create_completion_params.ResponseFormat | Omit = omit,
        stream: bool | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: Literal["none", "auto", "required"] | Omit = omit,
        tools: Iterable[chat_create_completion_params.Tool] | Omit = omit,
        top_logprobs: int | Omit = omit,
        top_p: float | Omit = omit,
        use_beam_search: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatCreateCompletionResponse:
        """Chat with a language model.

        This endpoint is consistent with the
        [OpenAI Chat Completions API](https://platform.openai.com/docs/api-reference/chat)
        and may be used with the OpenAI JS or Python SDK.

        Args:
          messages: A list of the previous chat messages for context.

          api_key_ref: If you are using an external inference provider like xAI or OpenAI, this field
              allows you to pass along a reference to your API key. After creating an
              [integration secret](https://developers.telnyx.com/api-reference/integration-secrets/create-a-secret)
              for you API key, pass the secret's `identifier` in this field.

          best_of: This is used with `use_beam_search` to determine how many candidate beams to
              explore.

          early_stopping: This is used with `use_beam_search`. If `true`, generation stops as soon as
              there are `best_of` complete candidates; if `false`, a heuristic is applied and
              the generation stops when is it very unlikely to find better candidates.

          enable_thinking: Whether to enable the thinking/reasoning phase for models that support it (e.g.,
              QwQ, Qwen3). When set to false, the model will skip the internal reasoning step
              and respond directly, which can reduce latency. Defaults to true.

          frequency_penalty: Higher values will penalize the model from repeating the same output tokens.

          guided_choice: If specified, the output will be exactly one of the choices.

          guided_json: Must be a valid JSON schema. If specified, the output will follow the JSON
              schema.

          guided_regex: If specified, the output will follow the regex pattern.

          length_penalty: This is used with `use_beam_search` to prefer shorter or longer completions.

          logprobs: Whether to return log probabilities of the output tokens or not. If true,
              returns the log probabilities of each output token returned in the `content` of
              `message`.

          max_tokens: Maximum number of completion tokens the model should generate.

          min_p: This is an alternative to `top_p` that
              [many prefer](https://github.com/huggingface/transformers/issues/27670). Must be
              in [0, 1].

          model: The language model to chat with.

          n: This will return multiple choices for you instead of a single chat completion.

          presence_penalty: Higher values will penalize the model from repeating the same output tokens.

          response_format: Use this is you want to guarantee a JSON output without defining a schema. For
              control over the schema, use `guided_json`.

          stream: Whether or not to stream data-only server-sent events as they become available.

          temperature: Adjusts the "creativity" of the model. Lower values make the model more
              deterministic and repetitive, while higher values make the model more random and
              creative.

          tools: The `function` tool type follows the same schema as the
              [OpenAI Chat Completions API](https://platform.openai.com/docs/api-reference/chat).
              The `retrieval` tool type is unique to Telnyx. You may pass a list of
              [embedded storage buckets](https://developers.telnyx.com/api-reference/embeddings/embed-documents)
              for retrieval-augmented generation.

          top_logprobs: This is used with `logprobs`. An integer between 0 and 20 specifying the number
              of most likely tokens to return at each token position, each with an associated
              log probability.

          top_p: An alternative or complement to `temperature`. This adjusts how many of the top
              possibilities to consider.

          use_beam_search: Setting this to `true` will allow the model to
              [explore more completion options](https://huggingface.co/blog/how-to-generate#beam-search).
              This is not supported by OpenAI.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/chat/completions",
            body=maybe_transform(
                {
                    "messages": messages,
                    "api_key_ref": api_key_ref,
                    "best_of": best_of,
                    "early_stopping": early_stopping,
                    "enable_thinking": enable_thinking,
                    "frequency_penalty": frequency_penalty,
                    "guided_choice": guided_choice,
                    "guided_json": guided_json,
                    "guided_regex": guided_regex,
                    "length_penalty": length_penalty,
                    "logprobs": logprobs,
                    "max_tokens": max_tokens,
                    "min_p": min_p,
                    "model": model,
                    "n": n,
                    "presence_penalty": presence_penalty,
                    "response_format": response_format,
                    "stream": stream,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                    "use_beam_search": use_beam_search,
                },
                chat_create_completion_params.ChatCreateCompletionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCreateCompletionResponse,
        )


class AsyncChatResource(AsyncAPIResource):
    """Generate text with LLMs"""

    @cached_property
    def with_raw_response(self) -> AsyncChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncChatResourceWithStreamingResponse(self)

    async def create_completion(
        self,
        *,
        messages: Iterable[chat_create_completion_params.Message],
        api_key_ref: str | Omit = omit,
        best_of: int | Omit = omit,
        early_stopping: bool | Omit = omit,
        enable_thinking: bool | Omit = omit,
        frequency_penalty: float | Omit = omit,
        guided_choice: SequenceNotStr[str] | Omit = omit,
        guided_json: Dict[str, object] | Omit = omit,
        guided_regex: str | Omit = omit,
        length_penalty: float | Omit = omit,
        logprobs: bool | Omit = omit,
        max_tokens: int | Omit = omit,
        min_p: float | Omit = omit,
        model: str | Omit = omit,
        n: float | Omit = omit,
        presence_penalty: float | Omit = omit,
        response_format: chat_create_completion_params.ResponseFormat | Omit = omit,
        stream: bool | Omit = omit,
        temperature: float | Omit = omit,
        tool_choice: Literal["none", "auto", "required"] | Omit = omit,
        tools: Iterable[chat_create_completion_params.Tool] | Omit = omit,
        top_logprobs: int | Omit = omit,
        top_p: float | Omit = omit,
        use_beam_search: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChatCreateCompletionResponse:
        """Chat with a language model.

        This endpoint is consistent with the
        [OpenAI Chat Completions API](https://platform.openai.com/docs/api-reference/chat)
        and may be used with the OpenAI JS or Python SDK.

        Args:
          messages: A list of the previous chat messages for context.

          api_key_ref: If you are using an external inference provider like xAI or OpenAI, this field
              allows you to pass along a reference to your API key. After creating an
              [integration secret](https://developers.telnyx.com/api-reference/integration-secrets/create-a-secret)
              for you API key, pass the secret's `identifier` in this field.

          best_of: This is used with `use_beam_search` to determine how many candidate beams to
              explore.

          early_stopping: This is used with `use_beam_search`. If `true`, generation stops as soon as
              there are `best_of` complete candidates; if `false`, a heuristic is applied and
              the generation stops when is it very unlikely to find better candidates.

          enable_thinking: Whether to enable the thinking/reasoning phase for models that support it (e.g.,
              QwQ, Qwen3). When set to false, the model will skip the internal reasoning step
              and respond directly, which can reduce latency. Defaults to true.

          frequency_penalty: Higher values will penalize the model from repeating the same output tokens.

          guided_choice: If specified, the output will be exactly one of the choices.

          guided_json: Must be a valid JSON schema. If specified, the output will follow the JSON
              schema.

          guided_regex: If specified, the output will follow the regex pattern.

          length_penalty: This is used with `use_beam_search` to prefer shorter or longer completions.

          logprobs: Whether to return log probabilities of the output tokens or not. If true,
              returns the log probabilities of each output token returned in the `content` of
              `message`.

          max_tokens: Maximum number of completion tokens the model should generate.

          min_p: This is an alternative to `top_p` that
              [many prefer](https://github.com/huggingface/transformers/issues/27670). Must be
              in [0, 1].

          model: The language model to chat with.

          n: This will return multiple choices for you instead of a single chat completion.

          presence_penalty: Higher values will penalize the model from repeating the same output tokens.

          response_format: Use this is you want to guarantee a JSON output without defining a schema. For
              control over the schema, use `guided_json`.

          stream: Whether or not to stream data-only server-sent events as they become available.

          temperature: Adjusts the "creativity" of the model. Lower values make the model more
              deterministic and repetitive, while higher values make the model more random and
              creative.

          tools: The `function` tool type follows the same schema as the
              [OpenAI Chat Completions API](https://platform.openai.com/docs/api-reference/chat).
              The `retrieval` tool type is unique to Telnyx. You may pass a list of
              [embedded storage buckets](https://developers.telnyx.com/api-reference/embeddings/embed-documents)
              for retrieval-augmented generation.

          top_logprobs: This is used with `logprobs`. An integer between 0 and 20 specifying the number
              of most likely tokens to return at each token position, each with an associated
              log probability.

          top_p: An alternative or complement to `temperature`. This adjusts how many of the top
              possibilities to consider.

          use_beam_search: Setting this to `true` will allow the model to
              [explore more completion options](https://huggingface.co/blog/how-to-generate#beam-search).
              This is not supported by OpenAI.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/chat/completions",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "api_key_ref": api_key_ref,
                    "best_of": best_of,
                    "early_stopping": early_stopping,
                    "enable_thinking": enable_thinking,
                    "frequency_penalty": frequency_penalty,
                    "guided_choice": guided_choice,
                    "guided_json": guided_json,
                    "guided_regex": guided_regex,
                    "length_penalty": length_penalty,
                    "logprobs": logprobs,
                    "max_tokens": max_tokens,
                    "min_p": min_p,
                    "model": model,
                    "n": n,
                    "presence_penalty": presence_penalty,
                    "response_format": response_format,
                    "stream": stream,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                    "use_beam_search": use_beam_search,
                },
                chat_create_completion_params.ChatCreateCompletionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCreateCompletionResponse,
        )


class ChatResourceWithRawResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

        self.create_completion = to_raw_response_wrapper(
            chat.create_completion,
        )


class AsyncChatResourceWithRawResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

        self.create_completion = async_to_raw_response_wrapper(
            chat.create_completion,
        )


class ChatResourceWithStreamingResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

        self.create_completion = to_streamed_response_wrapper(
            chat.create_completion,
        )


class AsyncChatResourceWithStreamingResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

        self.create_completion = async_to_streamed_response_wrapper(
            chat.create_completion,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ...types import number_reservation_list_params, number_reservation_create_params
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.number_reservation import NumberReservation
from ...types.reserved_phone_number_param import ReservedPhoneNumberParam
from ...types.number_reservation_create_response import NumberReservationCreateResponse
from ...types.number_reservation_retrieve_response import NumberReservationRetrieveResponse

__all__ = ["NumberReservationsResource", "AsyncNumberReservationsResource"]


class NumberReservationsResource(SyncAPIResource):
    """Number reservations"""

    @cached_property
    def actions(self) -> ActionsResource:
        """Number reservations"""
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> NumberReservationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return NumberReservationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NumberReservationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return NumberReservationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        customer_reference: str | Omit = omit,
        phone_numbers: Iterable[ReservedPhoneNumberParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberReservationCreateResponse:
        """
        Creates a Phone Number Reservation for multiple numbers.

        Args:
          customer_reference: A customer reference string for customer look ups.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/number_reservations",
            body=maybe_transform(
                {
                    "customer_reference": customer_reference,
                    "phone_numbers": phone_numbers,
                },
                number_reservation_create_params.NumberReservationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberReservationCreateResponse,
        )

    def retrieve(
        self,
        number_reservation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberReservationRetrieveResponse:
        """
        Gets a single phone number reservation.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not number_reservation_id:
            raise ValueError(
                f"Expected a non-empty value for `number_reservation_id` but received {number_reservation_id!r}"
            )
        return self._get(
            f"/number_reservations/{number_reservation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberReservationRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: number_reservation_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[NumberReservation]:
        """
        Gets a paginated list of phone number reservations.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[status],
              filter[created_at], filter[phone_numbers.phone_number],
              filter[customer_reference]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/number_reservations",
            page=SyncDefaultFlatPagination[NumberReservation],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    number_reservation_list_params.NumberReservationListParams,
                ),
            ),
            model=NumberReservation,
        )


class AsyncNumberReservationsResource(AsyncAPIResource):
    """Number reservations"""

    @cached_property
    def actions(self) -> AsyncActionsResource:
        """Number reservations"""
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncNumberReservationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNumberReservationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNumberReservationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncNumberReservationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        customer_reference: str | Omit = omit,
        phone_numbers: Iterable[ReservedPhoneNumberParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberReservationCreateResponse:
        """
        Creates a Phone Number Reservation for multiple numbers.

        Args:
          customer_reference: A customer reference string for customer look ups.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/number_reservations",
            body=await async_maybe_transform(
                {
                    "customer_reference": customer_reference,
                    "phone_numbers": phone_numbers,
                },
                number_reservation_create_params.NumberReservationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberReservationCreateResponse,
        )

    async def retrieve(
        self,
        number_reservation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberReservationRetrieveResponse:
        """
        Gets a single phone number reservation.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not number_reservation_id:
            raise ValueError(
                f"Expected a non-empty value for `number_reservation_id` but received {number_reservation_id!r}"
            )
        return await self._get(
            f"/number_reservations/{number_reservation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberReservationRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: number_reservation_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[NumberReservation, AsyncDefaultFlatPagination[NumberReservation]]:
        """
        Gets a paginated list of phone number reservations.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[status],
              filter[created_at], filter[phone_numbers.phone_number],
              filter[customer_reference]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/number_reservations",
            page=AsyncDefaultFlatPagination[NumberReservation],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    number_reservation_list_params.NumberReservationListParams,
                ),
            ),
            model=NumberReservation,
        )


class NumberReservationsResourceWithRawResponse:
    def __init__(self, number_reservations: NumberReservationsResource) -> None:
        self._number_reservations = number_reservations

        self.create = to_raw_response_wrapper(
            number_reservations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            number_reservations.retrieve,
        )
        self.list = to_raw_response_wrapper(
            number_reservations.list,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        """Number reservations"""
        return ActionsResourceWithRawResponse(self._number_reservations.actions)


class AsyncNumberReservationsResourceWithRawResponse:
    def __init__(self, number_reservations: AsyncNumberReservationsResource) -> None:
        self._number_reservations = number_reservations

        self.create = async_to_raw_response_wrapper(
            number_reservations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            number_reservations.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            number_reservations.list,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        """Number reservations"""
        return AsyncActionsResourceWithRawResponse(self._number_reservations.actions)


class NumberReservationsResourceWithStreamingResponse:
    def __init__(self, number_reservations: NumberReservationsResource) -> None:
        self._number_reservations = number_reservations

        self.create = to_streamed_response_wrapper(
            number_reservations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            number_reservations.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            number_reservations.list,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        """Number reservations"""
        return ActionsResourceWithStreamingResponse(self._number_reservations.actions)


class AsyncNumberReservationsResourceWithStreamingResponse:
    def __init__(self, number_reservations: AsyncNumberReservationsResource) -> None:
        self._number_reservations = number_reservations

        self.create = async_to_streamed_response_wrapper(
            number_reservations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            number_reservations.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            number_reservations.list,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        """Number reservations"""
        return AsyncActionsResourceWithStreamingResponse(self._number_reservations.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SiprecConnectorUpdateParams"]


class SiprecConnectorUpdateParams(TypedDict, total=False):
    host: Required[str]
    """Hostname/IPv4 address of the SIPREC SRS."""

    name: Required[str]
    """Name for the SIPREC connector resource."""

    port: Required[int]
    """Port for the SIPREC SRS."""

    app_subdomain: str
    """
    Subdomain to route the call when using Telnyx SRS (optional for non-Telnyx SRS).
    """
